@@ -8,7 +8,7 @@ use base 'Mojolicious::Controller';
 
 our $VERSION = '0.53';
 
-sub handle_request {
+sub dispatch {
     my $self = shift;
     
     my ($package, $method, @params, $id, $cross_domain, $data, $reply, $error);
@@ -31,10 +31,10 @@ sub handle_request {
     
     # cross-domain GET requests
     elsif ($self->req->method eq 'GET'){
-        $data           = $json->decode(
-                                $self->param('_ScriptTransport_data')
-                            );
-        $id             = $self->param('_ScriptTransport_id');
+        $data= $json->decode(
+            $self->param('_ScriptTransport_data')
+        );
+        $id = $self->param('_ScriptTransport_id') ;
         $cross_domain   = 1;
     }
     else{
@@ -46,13 +46,19 @@ sub handle_request {
         # "Transport error 0: Unknown status code" in qooxdoo
         return;
     }
-    
+        
+    if (not defined $id){
+        $self->app->log->fatal("This is not a JsonRPC request.");
+        return;
+    }
+
     # Getting available services from stash
     my $services = $self->stash('services');
-    
+
     # Check if desired service is available
     $package = $data->{service};
-    
+
+
     if (not exists $services->{$package}){
         $reply = $json->
             encode({
@@ -102,6 +108,28 @@ sub handle_request {
     
     # invocation of method in class according to request 
     eval{
+        # make sure there are not foreign signal handlers
+        # messing with our problems
+        local $SIG{__DIE__};
+        local $SIG{__WARN__};
+        if ($services->{$package}->can('_mojo_session')){
+            # initialize session if it does not exists yet
+            my $session = $self->stash->{'mojo.session'} ||= {};
+            $services->{$package}->_mojo_session($session);
+        }
+        if ($services->{$package}->can('_mojo_stash')){
+            # initialize session if it does not exists yet
+            $services->{$package}->_mojo_stash($self->stash);
+        }
+        if ($services->{$package}->can('_check_access')){
+            if ( not $services->{$package}->_check_access($method) ){
+	        die { 
+                 origin => 1, 
+	            message => "Permission denied. Access to method $method is denied.",
+                    code=> 1
+                }
+            }
+        }
         no strict 'refs';
         $reply = $services->{$package}->$method(@params);
     };
@@ -173,24 +201,25 @@ MojoX::Dispatcher::Qooxdoo::Jsonrpc - Dispatcher for Qooxdoo Json Rpc Calls
 =head1 SYNOPSIS
 
  # lib/your-application.pm
+
+ use base 'Mojolicious';
  
- use RpcService::Test;
- 
+ use RpcService;
+
  sub startup {
     my $self = shift;
     
     # instantiate all services
     my $services= {
-        Test => new RpcService::Test(),
+        Test => RpcService->new(),
         
     };
     
     
     # add a route to the Qooxdoo dispatcher and route to it
     my $r = $self->routes;
-    $r->route('/qooxdoo') ->
-            to('
-                Jsonrpc#handle_request', 
+    $r->route('/qooxdoo') -> to(
+                'Jsonrpc#dispatch', 
                 services    => $services, 
                 debug       => 0,
                 namespace   => 'MojoX::Dispatcher::Qooxdoo'
@@ -210,29 +239,37 @@ a (hopefully) valid json reply.
 =head1 EXAMPLE 
 
 This example exposes a service named "Test" in a folder "RpcService".
-The Mojo application is named "qooxdooserver". The scripts are in
+The Mojo application is named "QooxdooServer". The scripts are in
 the 'example' directory.
 First create this application using 
-"mojolicious generate app qooxdooserver".
+"mojolicious generate app QooxdooServer".
 
 Then, lets write the service:
 
-Change to the root directory "qooxdooserver" of your fresh 
+Change to the root directory "qooxdoo_server" of your fresh 
 Mojo-Application and make a dir named 'qooxdoo-services' 
 for the services you want to expose.
 
 Our "Test"-service could look like:
 
- package RpcService::Test;
+ package RpcService;
 
- sub new{
-    my $class = shift;
-    
-    my $object = {
-        
-    };
-    bless $object, $class;
-    return $object;
+ use base qw(Mojo::Base);
+
+ # if this attribute is created it will hold the mojo cookie session hash
+ __PACKAGE__->attr('_mojo_session');
+ # if this attribute exists it will provide access to the stash
+ __PACKAGE__->attr('_mojo_stash');
+ 
+ # optional access_check method the method is called right before the actual
+ # method is called but after the _mojo_session and _mojo_stash properties
+ # are assigned
+
+ sub _check_access {
+    my $self = shift;
+    my $method = shift;          
+    # check if we can access
+    return 1; # if ok
  }
 
  sub add{
@@ -295,7 +332,7 @@ Our "Test"-service could look like:
 
 Please create a constructor (like "new" here) which instantiates
 an object because we are going to use this in
-our 'lib/qooxdooserver.pm' below.
+our 'lib/QooxdooServer.pm' below.
 
 Notice the exception handling: You can die without or with a message 
 (see example above). 
@@ -306,12 +343,12 @@ Happy dying! :-)
 
 Now, lets write our application.
 Almost everything should have been prepared by Mojo when you invoked 
-"mojolicious generate app qooxdooserver" (see above).
+"mojolicious generate app QooxdooServer" (see above).
 
-Change to "lib/" and open "qooxdooserver.pm" in your favourite editor.
+Change to "lib/" and open "QooxdooServer.pm" in your favourite editor.
 Then add some lines to make it look like this:
 
- package qooxdooserver;
+ package QooxdooServer;
 
  use strict;
  use warnings;
@@ -336,7 +373,7 @@ Then add some lines to make it look like this:
     # to our little dispatcher.
     # change this at your own taste.
     $r->route('/qooxdoo')->to('
-        jsonrpc#handle_request', 
+        jsonrpc#dispatch', 
         services    => $services, 
         debug       => 0,
         namespace   => 'MojoX::Dispatcher::Qooxdoo'
@@ -346,8 +383,8 @@ Then add some lines to make it look like this:
 
  1;
 
-Now start your Mojo Server by issuing 'script/qooxdooserver daemon'. 
-If you want to change any options, type 'script/qooxdooserver help'. 
+Now start your Mojo Server by issuing 'script/QooxdooServer daemon'. 
+If you want to change any options, type 'script/QooxdooServer help'. 
 
 =head2 Security
 MojoX::Dispatcher::Qooxdoo::Jsonrpc only allows methods matching
